@@ -3,7 +3,7 @@
 import copy
 import json
 from json import JSONDecodeError
-from typing import Any, Union, Dict, List, Optional
+from typing import Any, Union, Dict, List, Optional, Callable, Tuple, Type
 
 from jsonpath_ng import Root, JSONPath
 from jsonpath_ng.exceptions import JsonPathParserError
@@ -13,37 +13,37 @@ from fakeredis import _helpers as helpers
 from fakeredis import _msgs as msgs
 from fakeredis._command_args_parsing import extract_args
 from fakeredis._commands import Key, command, delete_keys, CommandItem, Int, Float
+from fakeredis._helpers import SimpleString
 from fakeredis._zset import ZSet
 
 JsonType = Union[str, int, float, bool, None, Dict[str, Any], List[Any]]
 
 
-def _format_path(path) -> str:
-    if isinstance(path, bytes):
-        path = path.decode()
-    if path == ".":
+def _format_path(path: Union[bytes, str]) -> str:
+    path_str = path.decode() if isinstance(path, bytes) else path
+    if path_str == ".":
         return "$"
-    elif path.startswith("."):
-        return "$" + path
-    elif path.startswith("$"):
-        return path
+    elif path_str.startswith("."):
+        return "$" + path_str
+    elif path_str.startswith("$"):
+        return path_str
     else:
-        return "$." + path
+        return "$." + path_str
 
 
-def _parse_jsonpath(path: Union[str, bytes]):
-    path = _format_path(path)
+def _parse_jsonpath(path: Union[str, bytes]) -> JSONPath:
+    path_str: str = _format_path(path)
     try:
-        return parse(path)
+        return parse(path_str)
     except JsonPathParserError:
-        raise helpers.SimpleError(msgs.JSON_PATH_DOES_NOT_EXIST.format(path))
+        raise helpers.SimpleError(msgs.JSON_PATH_DOES_NOT_EXIST.format(path_str))
 
 
 def _path_is_root(path: JSONPath) -> bool:
-    return path == Root()
+    return path == Root()  # type: ignore
 
 
-def _dict_deep_merge(source: JsonType, destination: Dict) -> Dict:
+def _dict_deep_merge(source: JsonType, destination: Dict[str, Any]) -> Dict[str, Any]:
     """Deep merge of two dictionaries"""
     if not isinstance(source, dict):
         return destination
@@ -80,7 +80,12 @@ class JSONObject:
         return json.dumps(value, default=str).encode() if value is not None else None
 
 
-def _json_write_iterate(method, key, path_str, **kwargs):
+def _json_write_iterate(
+        method: Callable[[JsonType], Tuple[Optional[JsonType], Optional[Any], bool]],
+        key: CommandItem,
+        path_str: Union[str, bytes],
+        **kwargs: Any,
+) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
     """Implement json.* write commands.
     Iterate over values with path_str in key and running method to get new value for path item.
     """
@@ -94,7 +99,7 @@ def _json_write_iterate(method, key, path_str, **kwargs):
         )
 
     curr_value = copy.deepcopy(key.value)
-    res = list()
+    res: List[Optional[JsonType]] = list()
     for item in found_matches:
         new_value, res_val, update = method(item.value)
         if update:
@@ -113,7 +118,12 @@ def _json_write_iterate(method, key, path_str, **kwargs):
     return res
 
 
-def _json_read_iterate(method, key, *args, error_on_zero_matches=False):
+def _json_read_iterate(
+        method: Callable[[JsonType], Optional[Any]],
+        key: CommandItem,
+        *args: Any,
+        error_on_zero_matches: bool = False,
+) -> Union[List[Optional[Any]], Optional[Any]]:
     path_str = args[0] if len(args) > 0 else "$"
     if key.value is None:
         if path_str[0] == 36:
@@ -142,13 +152,13 @@ def _json_read_iterate(method, key, *args, error_on_zero_matches=False):
 class JSONCommandsMixin:
     """`CommandsMixin` for enabling RedisJSON compatibility in `fakeredis`."""
 
-    TYPES_EMPTY_VAL_DICT = {
+    TYPES_EMPTY_VAL_DICT: Dict[Type[object], Any] = {
         dict: {},
         int: 0,
         float: 0.0,
         list: [],
     }
-    TYPE_NAMES = {
+    TYPE_NAMES: Dict[Type[object], bytes] = {
         dict: b"object",
         int: b"integer",
         float: b"number",
@@ -158,7 +168,7 @@ class JSONCommandsMixin:
         str: b"string",
         bool: b"boolean",
         type(None): b"null",
-        ZSet: "zset",
+        ZSet: b"zset",
     }
 
     _db: helpers.Database
@@ -168,12 +178,12 @@ class JSONCommandsMixin:
 
     @staticmethod
     def _get_single(
-            key,
-            path_str: str,
+            key: CommandItem,
+            path_str: Union[str, bytes],
             always_return_list: bool = False,
             empty_list_as_none: bool = False,
     ) -> Any:
-        path = _parse_jsonpath(path_str)
+        path: JSONPath = _parse_jsonpath(path_str)
         path_value = path.find(key.value)
         val = [i.value for i in path_value]
         if empty_list_as_none and len(val) == 0:
@@ -188,7 +198,7 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_del(self, key, path_str) -> int:
+    def json_del(self, key: CommandItem, path_str: bytes) -> int:
         if key.value is None:
             return 0
 
@@ -210,7 +220,7 @@ class JSONCommandsMixin:
         return res
 
     @staticmethod
-    def _json_set(key: CommandItem, path_str: bytes, value: JsonType, *args):
+    def _json_set(key: CommandItem, path_str: bytes, value: JsonType, *args: Any) -> Optional[SimpleString]:
         path = _parse_jsonpath(path_str)
         if (
                 key.value is not None
@@ -234,7 +244,7 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_set(self, key, path_str: bytes, value: JsonType, *args):
+    def json_set(self, key: CommandItem, path_str: bytes, value: JsonType, *args: bytes) -> Optional[SimpleString]:
         """Set the JSON value at key `name` under the `path` to `obj`.
 
         For more information see `JSON.SET <https://redis.io/commands/json.set>`_.
@@ -247,14 +257,15 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_get(self, key, *args) -> Optional[bytes]:
+    def json_get(self, key: CommandItem, *args: bytes) -> Optional[bytes]:
         if key.value is None:
             return None
         paths = [arg for arg in args if not helpers.casematch(b"noescape", arg)]
         no_wrapping_array = len(paths) == 1 and paths[0][0] == ord(b".")
 
-        formatted_paths = [
-            _format_path(arg) for arg in args if not helpers.casematch(b"noescape", arg)
+        formatted_paths: List[str] = [
+            _format_path(arg) for arg in args
+            if not helpers.casematch(b"noescape", arg)
         ]
         path_values = [
             self._get_single(key, path, len(formatted_paths) > 1)
@@ -278,7 +289,7 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_mget(self, *args):
+    def json_mget(self, *args: bytes) -> List[Optional[bytes]]:
         if len(args) < 2:
             raise helpers.SimpleError(msgs.WRONG_ARGS_MSG6.format("json.mget"))
         path_str = args[-1]
@@ -299,10 +310,10 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_toggle(self, key, *args):
+    def json_toggle(self, key: CommandItem, *args: bytes) -> Union[List[Optional[bool]], Optional[bool]]:
         if key.value is None:
             raise helpers.SimpleError(msgs.JSON_KEY_NOT_FOUND)
-        path_str = args[0] if len(args) > 0 else "$"
+        path_str = args[0] if len(args) > 0 else b"$"
         path = _parse_jsonpath(path_str)
         found_matches = path.find(key.value)
 
@@ -329,14 +340,10 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_clear(
-            self,
-            key,
-            *args,
-    ):
+    def json_clear(self, key: CommandItem, *args: bytes) -> int:
         if key.value is None:
             raise helpers.SimpleError(msgs.JSON_KEY_NOT_FOUND)
-        path_str = args[0] if len(args) > 0 else "$"
+        path_str: bytes = args[0] if len(args) > 0 else b"$"
         path = _parse_jsonpath(path_str)
         found_matches = path.find(key.value)
         curr_value = copy.deepcopy(key.value)
@@ -356,12 +363,13 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_strappend(self, key, path_str, *args):
+    def json_strappend(self, key: CommandItem, path_str: bytes, *args: bytes
+                       ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
         if len(args) == 0:
             raise helpers.SimpleError(msgs.WRONG_ARGS_MSG6.format("json.strappend"))
         addition = JSONObject.decode(args[0])
 
-        def strappend(val):
+        def strappend(val: JsonType) -> Tuple[Optional[JsonType], Optional[int], bool]:
             if type(val) is str:
                 new_value = val + addition
                 return new_value, len(new_value), True
@@ -372,20 +380,18 @@ class JSONCommandsMixin:
 
     @command(
         name="JSON.ARRAPPEND",
-        fixed=(
-                Key(),
-                bytes,
-        ),
+        fixed=(Key(), bytes),
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_arrappend(self, key, path_str, *args):
+    def json_arrappend(self, key: CommandItem, path_str: bytes, *args: bytes
+                       ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
         if len(args) == 0:
             raise helpers.SimpleError(msgs.WRONG_ARGS_MSG6.format("json.arrappend"))
 
         addition = [JSONObject.decode(item) for item in args]
 
-        def arrappend(val):
+        def arrappend(val: JsonType) -> Tuple[Optional[JsonType], Optional[int], bool]:
             if type(val) is list:
                 new_value = val + addition
                 return new_value, len(new_value), True
@@ -400,13 +406,15 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_arrinsert(self, key, path_str, index, *args):
+    def json_arrinsert(
+            self, key: CommandItem, path_str: bytes, index: int, *args: bytes
+    ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
         if len(args) == 0:
             raise helpers.SimpleError(msgs.WRONG_ARGS_MSG6.format("json.arrinsert"))
 
         addition = [JSONObject.decode(item) for item in args]
 
-        def arrinsert(val):
+        def arrinsert(val: JsonType) -> Tuple[Optional[JsonType], Optional[int], bool]:
             if type(val) is list:
                 new_value = val[:index] + addition + val[index:]
                 return new_value, len(new_value), True
@@ -415,17 +423,14 @@ class JSONCommandsMixin:
 
         return _json_write_iterate(arrinsert, key, path_str)
 
-    @command(
-        name="JSON.ARRPOP",
-        fixed=(Key(),),
-        repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
-    def json_arrpop(self, key, *args):
-        path_str = args[0] if len(args) > 0 else "$"
+    @command(name="JSON.ARRPOP", fixed=(Key(),), repeat=(bytes,), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
+    def json_arrpop(
+            self, key: CommandItem, *args: bytes
+    ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
+        path_str: Union[bytes, str] = args[0] if len(args) > 0 else "$"
         index = Int.decode(args[1]) if len(args) > 1 else -1
 
-        def arrpop(val):
+        def arrpop(val: JsonType) -> Tuple[Optional[JsonType], Optional[bytes], bool]:
             if type(val) is list and len(val) > 0:
                 ind = index if index < len(val) else -1
                 res = val.pop(ind)
@@ -441,12 +446,12 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_arrtrim(self, key, *args):
-        path_str = args[0] if len(args) > 0 else "$"
+    def json_arrtrim(self, key: CommandItem, *args: bytes) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
+        path_str: bytes = args[0] if len(args) > 0 else b"$"
         start = Int.decode(args[1]) if len(args) > 1 else 0
         stop = Int.decode(args[2]) if len(args) > 2 else None
 
-        def arrtrim(val):
+        def arrtrim(val: JsonType) -> Tuple[Optional[JsonType], Optional[int], bool]:
             if type(val) is list:
                 start_ind = min(start, len(val))
                 stop_ind = len(val) if stop is None or stop == -1 else stop + 1
@@ -465,10 +470,12 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_numincrby(self, key, path_str, inc_by, *_):
-        def numincrby(val):
-            if type(val) in {int, float}:
-                new_value = val + inc_by
+    def json_numincrby(
+            self, key: CommandItem, path_str: bytes, inc_by: float, *_: bytes
+    ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
+        def numincrby(val: Optional[JsonType]) -> Tuple[Optional[JsonType], Optional[float], bool]:
+            if val is not None and type(val) in {int, float}:
+                new_value = val + inc_by  # type: ignore
                 return new_value, new_value, True
             else:
                 return None, None, False
@@ -481,10 +488,12 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_nummultby(self, key, path_str, mult_by, *_):
-        def nummultby(val):
+    def json_nummultby(
+            self, key: CommandItem, path_str: bytes, mult_by: float, *_: bytes
+    ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
+        def nummultby(val: Optional[JsonType]) -> Tuple[Optional[JsonType], Optional[float], bool]:
             if type(val) in {int, float}:
-                new_value = val * mult_by
+                new_value = val * mult_by  # type: ignore
                 return new_value, new_value, True
             else:
                 return None, None, False
@@ -498,13 +507,15 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_arrindex(self, key, path_str, encoded_value, *args):
+    def json_arrindex(
+            self, key: CommandItem, path_str: bytes, encoded_value: bytes, *args: bytes
+    ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
         start = max(0, Int.decode(args[0]) if len(args) > 0 else 0)
         end = Int.decode(args[1]) if len(args) > 1 else -1
         end = end if end > 0 else -1
         expected_value = JSONObject.decode(encoded_value)
 
-        def check_index(value):
+        def check_index(value: JsonType) -> Optional[int]:
             if type(value) is not list:
                 return None
             try:
@@ -516,27 +527,19 @@ class JSONCommandsMixin:
             except StopIteration:
                 return -1
 
-        return _json_read_iterate(
-            check_index, key, path_str, *args, error_on_zero_matches=True
-        )
+        return _json_read_iterate(check_index, key, path_str, *args, error_on_zero_matches=True)
 
     @command(name="JSON.STRLEN", fixed=(Key(),), repeat=(bytes,))
-    def json_strlen(self, key, *args):
-        return _json_read_iterate(
-            lambda val: len(val) if type(val) is str else None, key, *args
-        )
+    def json_strlen(self, key: CommandItem, *args: bytes) -> Union[List[Optional[int]], Optional[int]]:
+        return _json_read_iterate(lambda val: len(val) if type(val) is str else None, key, *args)
 
     @command(name="JSON.ARRLEN", fixed=(Key(),), repeat=(bytes,))
-    def json_arrlen(self, key, *args):
-        return _json_read_iterate(
-            lambda val: len(val) if type(val) is list else None, key, *args
-        )
+    def json_arrlen(self, key: CommandItem, *args: bytes) -> Union[List[Optional[int]], Optional[int]]:
+        return _json_read_iterate(lambda val: len(val) if type(val) is list else None, key, *args)
 
     @command(name="JSON.OBJLEN", fixed=(Key(),), repeat=(bytes,))
-    def json_objlen(self, key, *args):
-        return _json_read_iterate(
-            lambda val: len(val) if type(val) is dict else None, key, *args
-        )
+    def json_objlen(self, key: CommandItem, *args: bytes) -> Union[List[Optional[int]], Optional[int]]:
+        return _json_read_iterate(lambda val: len(val) if type(val) is dict else None, key, *args)
 
     @command(
         name="JSON.TYPE",
@@ -544,22 +547,13 @@ class JSONCommandsMixin:
         repeat=(bytes,),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_type(
-            self,
-            key,
-            *args,
-    ):
-        return _json_read_iterate(
-            lambda val: self.TYPE_NAMES.get(type(val), None), key, *args
-        )
+    def json_type(self, key: CommandItem, *args: bytes) -> Union[List[Optional[bytes]], Optional[bytes]]:
+        return _json_read_iterate(lambda val: self.TYPE_NAMES.get(type(val), None), key, *args)
 
     @command(name="JSON.OBJKEYS", fixed=(Key(),), repeat=(bytes,))
-    def json_objkeys(self, key, *args):
+    def json_objkeys(self, key: CommandItem, *args: bytes) -> Union[List[Optional[bytes]], Optional[bytes]]:
         return _json_read_iterate(
-            lambda val: [i.encode() for i in val.keys()] if type(val) is dict else None,
-            key,
-            *args,
-        )
+            lambda val: [i.encode() for i in val.keys()] if type(val) is dict else None, key, *args)
 
     @command(
         name="JSON.MSET",
@@ -567,7 +561,7 @@ class JSONCommandsMixin:
         repeat=(Key(), bytes, JSONObject),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_mset(self, *args):
+    def json_mset(self, *args: Any) -> SimpleString:
         if len(args) < 3 or len(args) % 3 != 0:
             raise helpers.SimpleError(msgs.WRONG_ARGS_MSG6.format("json.mset"))
         for i in range(0, len(args), 3):
@@ -581,7 +575,7 @@ class JSONCommandsMixin:
         repeat=(),
         flags=msgs.FLAG_LEAVE_EMPTY_VAL,
     )
-    def json_merge(self, key, path_str: bytes, value: JsonType):
+    def json_merge(self, key: CommandItem, path_str: bytes, value: JsonType) -> SimpleString:
         path: JSONPath = _parse_jsonpath(path_str)
         if (
                 key.value is not None
